#include <QtGlobal>
#include <QtGui>
// #include <QFileDialog>
// #include <QtDebug>
// #include <QDir>
// #include <QThread>
// #include <QDialog>
// #include <QProcess>
// #include <QByteArray>

#include "convert.h"






Convert::Convert()
{
	setupUi( this );

	messages->clear();			// efface la fenêtre de messages
	fileList->clear();			// efface la liste de fichiers
	fileList->setIconSize(QSize(24, 24));	// taille des icônes de la liste de fichiers

	progression->reset();		// barre de progression mise à zéro
	whereDir->setText("???");	// le répertoire où on va stocker les résultats est mis à une valeur incorrecte
								// pour forcer l'utilisateur à le renseigner

	scripts->setChecked(true);	// par défaut, case "Scripts" cochée (demande de Skiper)
	allEnabled (true);			// déverrouillage des boutons (seront verrouillés lors des conversion)
	testStartEnable();			// vérifie si on peut lancer la conversion
};

Convert::~Convert()
{
	// destructeur inutile : rien à faire
};


void Convert::on_selectPackages_clicked()
{
	// ouverture de la boîte de dialogue de sélection multiple de fichiers. La liste des fichiers est dans "files"
	files = QFileDialog::getOpenFileNames(this, tr("Sélectionnez les fichiers à convertir"), "", tr("Paquetages (*.rpm *.deb *.slp *.tar.gz *.tgz)"));
	fileList->clear();	// efface la liste de fichier éventuellement existante

	// création de la liste de fichiers
	foreach (QString file, files)	// pour chaque fichier dans "files"
	{
		QIcon icon;

		// si le nom de fichier finit par "rpm" on sélectionne l'icône rpm
		if (file.endsWith("rpm"))
		{
			icon = QIcon(QString::fromUtf8(":/roswell/icons/rpm.png"));
		}

		// etc...
		if (file.endsWith("deb"))
		{
			icon = QIcon(QString::fromUtf8(":/roswell/icons/deb.png"));
		}

		if (file.endsWith("tar.gz") || file.endsWith( "tgz"))
		{
			icon = QIcon(QString::fromUtf8(":/roswell/icons/tar-gz.png"));
		}

		// création de l'élément de la liste et affectation de l'icône
		QListWidgetItem *item = new QListWidgetItem(icon, file);
		// l'élément est ajouté à la liste
		fileList->addItem(item);
	}

	testStartEnable();// vérifie si on peut lancer la conversion
}

void Convert::on_where_clicked()
{
	// ouverture de la boîte de dialogue Qt pour sélectionner un répertoire
	whereDir->setText(QFileDialog::getExistingDirectory (this, tr("Choisissez la destination")));
	testStartEnable();
}


void Convert::on_start_clicked()
{
	// préparation de la ligne de commande
	// ce sera alien --to-xxx <--scripts> <fichier>

	// si la case scripts est cochée, on prépare l'option --scripts
	QString doScripts(scripts->isChecked() ? " --scripts " : " ");

	QStringList commandLines;	// variable-liste qui contiendra les débuts des lignes de commandes : "alien --to-xxx <--scripts> "

	if (RPMformat->isChecked())
	{
		commandLines << "alien --to-rpm" + doScripts;
	}

	if (DEBformat->isChecked())
	{
		commandLines << "alien --to-deb" + doScripts;
	}

	if (TARGZformat->isChecked())
	{
		commandLines << "alien --to-tgz" + doScripts;
	}

	if (SLPformat->isChecked())
	{
		commandLines << "alien --to-slp" + doScripts;
	}

	// effacement de la fenêtre des messages
	displayMessage(BLUE, tr("==== NOUVEAU TRAITEMENT ====\n"));	// affichage du texte en bleu dans la fenêtre des messages

	// réglage de la barre de progression
	// le nombre d'étapes = le nombre de fichiers à traiter * nombre de formats de sortie
	progression->setRange (0, commandLines.count() * files.count() );
	progression->reset();	// RAZ de la barre de progression

	// désactive certains boutons pendant le traitement
	// pour éviter que l'utilisateur aille cliquer partout
	// et mettre le souk
	allEnabled(false);

	int count = 0;		// compteur d'étapes
	int rowNumber = 0;	// pointeur de ligne

	// parcours imbriqué des listes
	foreach (QString file, files)
	{
		fileList->setCurrentRow(rowNumber);	// cosmétique : le nom du fichier en cours de traitement est sélectionné en haut

		foreach (QString commandLine, commandLines)
		{
			// petits calculs pour rendre les traces plus compréhensibles
			bool isRPM = commandLine.contains("--to-rpm");
			bool isDEB = commandLine.contains("--to-deb");
			bool isTGZ = commandLine.contains("--to-tgz");
			bool isSLP = commandLine.contains("--to-slp");

			// prépare le nom du format qui va s'afficher "Conversion du paquetage %1 au format XXXX..."
			QString formatName (isRPM ? "RPM" : isDEB ? "DEB" : isTGZ ? "TGZ" : isSLP ? "SLP" : "");

			displayMessage(BLACK, "-----------------------------");
			displayMessage(BLACK, tr("Conversion du paquetage %1 au format %2...").arg(file).arg(formatName));

			// test : vérifier qu'on ne va pas essayer de transformer un .rpm en... .rpm ! (Skiper)
			bool noUse =  (file.endsWith("rpm") && isRPM);
			noUse |= (file.endsWith("deb") && isDEB);
			noUse |= (file.endsWith("slp") && isSLP);
			noUse |= ((file.endsWith("tar.gz") || file.endsWith("tgz")) && isTGZ);

			if (noUse)
			{
				displayMessage(BLACK, tr("Pas de conversion d'un fichier vers lui-même !"));
			}
			else
			{
				// mini-astuce pour que l'affichage se rafraîchisse : on va voir s'il n'y a pas d'événement non traité
				qApp->processEvents();

				QString out, err;

				execute(commandLine + file, out, err);	// exécution de la ligne de commande et retour des résultats, sortie normale et erreur

				// on enrichit les traces
				displayMessage(RED, err);
				displayMessage(GREEN, out);

				qApp->processEvents();
			}

			// après traitement faire avancer la barre de progression
			count++;
			progression->setValue (count);
		}

		rowNumber++;	// ligne suivante
	}

	// réactive les boutons
	allEnabled(true);
};

void Convert::execute (QString commandLine, QString& out, QString& err)
{
	// on va lancer la ligne de commande qui est dans commandLine

	QProcess process;

	//on demande à Alien de se positionner dans le répertoire de travail
	// là où il doit fabriquer les paquetages
	process.setWorkingDirectory(whereDir->text());
	process.start (commandLine);	// ligne centrale : on appelle Alien - tout le programme est là pour ça :)

	if (!process.waitForFinished())	// on attend que le process ait fini
	{
		// le process n'a pas démarré
		out = QString();
		err = tr("Le processus n'a pas démarré\nVérifiez que Alien est installé.");
	}
	else
	{
		out = QString::fromUtf8(process.readAllStandardOutput());
		err = QString::fromUtf8(process.readAllStandardError());
	}
}

void Convert::allEnabled (bool enabled)
{
	// active ou désactive les boutons pendant le traitement
	selectPackages->setEnabled(enabled);
	start->setEnabled(enabled);
	quit->setEnabled(enabled);


	// modifie le curseur de la souris
	if (enabled)
	{
		QApplication::restoreOverrideCursor();
		progression->hide();	// cache la barre de progression
	}
	else
	{
		progression->show();	// montre la barre de progression
		QApplication::setOverrideCursor(Qt::WaitCursor);
	}
}

void Convert::displayMessage (QColor color, QString messOut)
{
	// messages dans le fenêtre du bas

	messages->moveCursor(QTextCursor::End);
	messages->setTextColor(color);
	messages->insertPlainText(messOut + "\n");
}


void Convert::testStartEnable()
{
	// test : doit-on activer le bouton start ??
	// on ne l'active que...

	// 1/ si au moins un format est sélectionné
	bool enable = RPMformat->isChecked();
	enable |= DEBformat->isChecked();
	enable |= SLPformat->isChecked();
	enable |= TARGZformat->isChecked();

	// 2/ si la liste des fichiers n'est pas vide
	enable &= !files.isEmpty();

	// 3/ et si le répertoire de travail existe
	enable &= QDir(whereDir->text()).exists();

	start->setEnabled(enable);
}

void Convert::on_help_toggled()
{
	// help et traces sont des boutons à 2 positions : checked ou unchecked

	if (help->isChecked())	// si aide est enfoncé on "désenfonce" trace
	{
		traces->setChecked(false);
	}

	// si help est "checked" on affiche la page d'aide (1), sinon la page principale (0)
	pages->setCurrentIndex(help->isChecked() ? 1 : 0);

};

// idem pour traces

void Convert::on_traces_toggled()
{
	if (traces->isChecked())	// si traces est enfoncé on "désenfonce" aide
	{
		help->setChecked(false);
	}

	pages->setCurrentIndex(traces->isChecked() ? 2 : 0);
};

