#include <stdio.h>
#include <stdlib.h>
#include <QTranslator>
#include <QLibraryInfo>
#include "convert.h"
#include <QtGui>

void myMessageOutput(QtMsgType type, const char *msg)
{
    switch (type)
    {
		case QtDebugMsg:
			fprintf(stdout, "%s\n", msg);
			break;
		case QtWarningMsg:
			fprintf(stderr, "Avertissement : %s\n", msg);
			break;
		case QtCriticalMsg:
			fprintf(stderr, "Erreur critique : %s\n", msg);
			break;
		case QtFatalMsg:
			fprintf(stderr, "Erreur fatale : %s\n", msg);
			abort();
    }
}

QString getenv(QString var)
{
	if (!var.endsWith("=")) var += "=";
	
	QStringList env( QProcess::systemEnvironment() );	// récupère l'environnement sous forme de liste de chaînes
	// puis le parcourt
	foreach ( QString chaine, env ) {
		if ( chaine.startsWith(var) ) {	// la chaîne d'environnement débute par "<var>=" 
			QString formattedEnv( chaine.mid(var.size()) );	// prend tout ce qui est après le signe '='
			if (!formattedEnv.endsWith("/")) {
				formattedEnv += "/";
			}
			return formattedEnv;
		}
	}
	
	return QString();
};




int main( int argc, char *argv[] )
{
	//	***************************	//
	//	Initialisations importantes	//
	//	***************************	//
	
	qInstallMsgHandler(myMessageOutput);
	QApplication::setDesktopSettingsAware(true);
	QApplication app( argc, argv );
	qDebug() << QCoreApplication::applicationDirPath();

	// pour avoir les boutons des boîtes de dialogue dans la langue locale (fr par défaut ou en, ou es, ou...)
	QTranslator qtTranslator;
	qtTranslator.load("qt_" + QLocale::system().name(),QLibraryInfo::location(QLibraryInfo::TranslationsPath));
	app.installTranslator(&qtTranslator);

	// pour avoir l'appli localisée
	QTranslator myappTranslator;
	
	// commence par chercher le fichier de langue dans le répertoire roswell/i18n
	bool ok = false;
	QString user(getenv("USER"));
	qDebug()<< user;
	QString env(getenv("DATADIR"));
	if (!env.isEmpty()) {
		ok = myappTranslator.load( "roswell_" + QLocale::system().name(), env + "roswell/i18n" );
	}
	// le cherche ensuite dans les ressources
	if (!ok) ok = myappTranslator.load( ":/i18n/i18n/roswell_" + QLocale::system().name());
	
	// le cherche dans le répertoire i18n de l'appli
	if (!ok) ok = myappTranslator.load( "roswell_" + QLocale::system().name(), QCoreApplication::applicationDirPath() + "/i18n" );
	
	// si trouvé installe les traductions
	if (ok) app.installTranslator(&myappTranslator);

	// pour les settings
	app.setOrganizationName( "PapaJaac" );
	app.setApplicationName( "roswell" );
	/********************************************************/


	// accès au formulaire créé par Qt4 Designer et lecture des "settings"
	Convert *dial = new Convert();

	// afficher le formulaire
	return dial->exec() ;

} // main

